# Generated by Django 2.0.7 on 2019-04-27 09:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('seo', '0003_auto_20190427_0906'),
    ]

    operations = [
        migrations.AddField(
            model_name='errorrecord',
            name='referrer',
            field=models.CharField(blank=True, max_length=1024, verbose_name='Referrer'),
        )
    ]




from django.db import models
from django.template import Template
from django.utils.translation import ugettext_lazy as _


class PageMeta(models.Model):

    META_ROBOTS_CHOICES = (
        ('index, follow', _('Index, Follow')),
        ('noindex, nofollow', _('No Index, No Follow')),
        ('index, nofollow', _('Index, No Follow')),
        ('noindex, follow', _('No Index, Follow')),
    )

    FIELDS_FOR_RENDERING = [
        'title', 'keywords', 'description', 'breadcrumb', 'header']

    url = models.CharField(
        _('URL'), max_length=255, blank=False, unique=True, db_index=True)

    title = models.CharField(_('Title'), max_length=68, blank=False)

    keywords = models.CharField(_('Keywords'), max_length=100, blank=True)

    description = models.CharField(
        _('Description'), max_length=155, blank=True)

    breadcrumb = models.CharField(_('Breadcrumb'), max_length=100, blank=True)

    header = models.CharField(_('Header'), max_length=100, blank=True)

    robots = models.CharField(
        _('Robots'), max_length=30, blank=True, default='index, follow',
        choices=META_ROBOTS_CHOICES)

    def render(self, context):

        for field in self.FIELDS_FOR_RENDERING:

            value = Template(getattr(self, field)).render(context)

            setattr(self, 'printable_' + field, value)

    def __str__(self):
        return '{} - {}'.format(self.url, self.title)

    class Meta:
        verbose_name = _('Page meta')
        verbose_name_plural = _('Pages meta')


class RedirectRecord(models.Model):

    old_path = models.CharField(
        _('Old path'), max_length=1024, db_index=True, unique=True)

    new_path = models.CharField(_('New path'), max_length=1024)

    def __str__(self):
        return self.old_path

    class Meta:
        verbose_name = _('Redirect record')
        verbose_name_plural = _('Redirect records')


class ErrorRecord(models.Model):

    path = models.CharField(_('Path'), max_length=1024, db_index=True)

    referrer = models.CharField(_('Referrer'), max_length=1024, blank=True)

    status_code = models.IntegerField(_('Status code'))

    created = models.DateTimeField(_('Creation date'), auto_now_add=True)

    method = models.CharField(_('Method'), max_length=100)

    def __str__(self):
        return '[{}] {}'.format(str(self.status_code).upper(), self.path)

    @classmethod
    def create(cls, request, status_code):
        print(request.META)
        return cls.objects.create(
            path=request.get_full_path(),
            status_code=status_code,
            method=request.method,
            referrer=request.META.get('HTTP_REFERER', ''))

    class Meta:
        verbose_name = _('Error record')
        verbose_name_plural = _('Error records')



# Generated by Django 2.0.7 on 2019-04-27 09:06

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('seo', '0002_auto_20190427_0832'),
    ]

    operations = [
        migrations.CreateModel(
            name='ErrorRecord',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('path', models.CharField(db_index=True, max_length=1024, verbose_name='Path')),
                ('status_code', models.IntegerField(verbose_name='Status code')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Creation date')),
            ],
            options={
                'verbose_name': 'Error record',
                'verbose_name_plural': 'Error records',
            },
        )
    ]



# Generated by Django 2.0.7 on 2019-04-27 08:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('seo', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='RedirectRecord',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('old_path', models.CharField(db_index=True, max_length=1024, unique=True, verbose_name='Old path')),
                ('new_path', models.CharField(max_length=1024, verbose_name='New path')),
            ],
            options={
                'verbose_name': 'Redirect record',
                'verbose_name_plural': 'Redirect records',
            },
        )
    ]



# Generated by Django 2.0.7 on 2019-04-27 10:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('seo', '0004_auto_20190427_0953'),
    ]

    operations = [
        migrations.AddField(
            model_name='errorrecord',
            name='method',
            field=models.CharField(default='get', max_length=100, verbose_name='Method'),
            preserve_default=False,
        )
    ]




from importlib import import_module

from django.apps import apps
from django.contrib import admin

from import_export.resources import ModelResource
from import_export.admin import ImportExportMixin, ExportActionMixin

from seo.models import PageMeta, RedirectRecord, ErrorRecord


def _get_page_meta_admin_base_class():

    if apps.is_installed('modeltranslation'):
        return import_module('modeltranslation.admin').TranslationAdmin

    return admin.ModelAdmin


class PageMetaResource(ModelResource):
    class Meta:
        model = PageMeta
        exclude = ('id', )


class ImportExportAdmin(
        ImportExportMixin,
        ExportActionMixin):
    actions_on_bottom = False


@admin.register(PageMeta)
class PageMetaAdmin(ImportExportAdmin, _get_page_meta_admin_base_class()):
    resource_class = PageMetaResource
    list_display = ['url', 'title', 'robots']
    list_editable = ['robots']
    list_filter = ['robots']
    search_fields = ['url', 'title']


@admin.register(RedirectRecord)
class RedirectRecordAdmin(ImportExportAdmin, admin.ModelAdmin):

    list_display = ['id', 'old_path', 'new_path']
    list_display_links = ['old_path', 'new_path']
    search_fields = ['old_path', 'new_path']


@admin.register(ErrorRecord)
class ErrorRecordAdmin(ExportActionMixin, admin.ModelAdmin):

    list_display = [
        'id', 'path', 'method', 'status_code', 'referrer', 'created']
    list_display_links = ['path']
    search_fields = ['path']
    list_filter = ['status_code']




from django.shortcuts import render, redirect
from django.utils.translation import ugettext_lazy as _

from seo.models import RedirectRecord, ErrorRecord


def _render_error_page(request, code, message, template_name='error.html'):
    context = {'code': code, 'message': message}
    return render(request, template_name, context, status=code)


def page_not_found(request, **kwargs):

    path = request.path

    try:
        record = RedirectRecord.objects.get(old_path=path)
        return redirect(record.new_path, permanent=True)
    except RedirectRecord.DoesNotExist:
        pass

    if not path.startswith('/static/') and not path.startswith('/media/'):
        ErrorRecord.create(request, 404)

    return _render_error_page(request, 404, _('Page not found'))


def server_error(request, **kwargs):
    return _render_error_page(request, 500, _('Server error'))


def permission_denied(request, **kwargs):
    return _render_error_page(request, 403, _('Permission denied'))


def bad_request(request, **kwargs):
    return _render_error_page(request, 400, _('Bad request'))
